#include "AudioDialogData.h"

/*
 *  Constructs a AudioDialogData as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
AudioDialogData::AudioDialogData( QWidget* parent ) 
{
    setupUi(this); // this sets up GUI  
}

/*
 *  Destroys the object and frees any allocated resources
 */
AudioDialogData::~AudioDialogData()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void AudioDialogData::languageChange()
{
    setWindowTitle( tr( "Audio configuration" ) );
    bitsGrp->setTitle( tr( "Sample precision" ) );
    bit16Btn->setText( tr( "16-bit" ) );
    bit8Btn->setText( tr( "8-bit" ) );
    channelsGrp->setTitle( tr( "Channels" ) );
    monoBtn->setText( tr( "Mono" ) );
    stereoBtn->setText( tr( "Stereo" ) );
    cancelButton->setText( tr( "&Cancel" ) );
    cancelButton->setShortcut( QKeySequence( tr( "Alt+C" ) ) );
    okButton->setText( tr( "&OK" ) );
    okButton->setShortcut( QKeySequence( tr( "Alt+O" ) ) );
    freqBox->setTitle( tr( "Frequency" ) );
    frequencyBox->clear();
    frequencyBox->addItem( tr( "8000" ) );
    frequencyBox->addItem( tr( "11025" ) );
    frequencyBox->addItem( tr( "16500" ) );
    frequencyBox->addItem( tr( "22050" ) );
    frequencyBox->addItem( tr( "27500" ) );
    frequencyBox->addItem( tr( "32000" ) );
    frequencyBox->addItem( tr( "37500" ) );
    frequencyBox->addItem( tr( "44100" ) );
    frequencyBox->addItem( tr( "48000" ) );
    frequencyBox->setCurrentIndex( 3 );
    bufferBox->setTitle( tr( "Buffer Size" ) );
    bufferSizeBox->clear();
    bufferSizeBox->addItem( tr( "256" ) );
    bufferSizeBox->addItem( tr( "512" ) );
    bufferSizeBox->addItem( tr( "1024" ) );
    bufferSizeBox->addItem( tr( "2048" ) );
    bufferSizeBox->addItem( tr( "4096" ) );
    bufferSizeBox->addItem( tr( "8192" ) );
    bufferSizeBox->addItem( tr( "16384" ) );
    bufferSizeBox->addItem( tr( "32768" ) );
    bufferSizeBox->setCurrentIndex( 5 );
    driverGrp->setTitle( tr( "Driver" ) );
    buttonGroupFormat->setTitle( tr( "Sample Format" ) );
    unsignedBtn->setText( tr( "Unsigned" ) );
    signedBtn->setText( tr( "Signed" ) );
}

void AudioDialogData::setDriver(const QString&)
{
    qWarning( "AudioDialogData::setDriver(const QString&): Not implemented yet" );
}

void AudioDialogData::setBitsPerSample(int)
{
    qWarning( "AudioDialogData::setBitsPerSample(int): Not implemented yet" );
}

void AudioDialogData::setChannels(int)
{
    qWarning( "AudioDialogData::setChannels(int): Not implemented yet" );
}

void AudioDialogData::setFrequency(int)
{
    qWarning( "AudioDialogData::setFrequency(int): Not implemented yet" );
}

void AudioDialogData::setBufferSize(int)
{
    qWarning( "AudioDialogData::setBufferSize(int): Not implemented yet" );
}

void AudioDialogData::setPrecision(int)
{
    qWarning( "AudioDialogData::setPrecision(int): Not implemented yet" );
}

void AudioDialogData::setEncoding(int)
{
    qWarning( "AudioDialogData::setEncoding(int): Not implemented yet" );
}

