/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "EmuDialogNew.h"
#include "wrapper/sid/SidDrivers.h"
#include "wrapper/sid/EmuConfigImpl.h"

/*
 *  Constructs a EmuDialogData as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
EmuDialogNew::EmuDialogNew( QWidget* parent )
    : QDialog( parent)
{
    setupUi(this); // this sets up GUI
    
   /// setWindowIcon(*mainIcon);
    
    // Fill combo box with audio driver names and activate tabs
    SidDrivers::List::iterator it;
    for ( it = SidDrivers::begin(); it != SidDrivers::end(); it++ )
    {
        libraryBox->insertItem( -1, it->name );
        switch (it->library) {
          case EmuConfig::EMPTY:
            tabNull->setEnabled(true);    
            break;
          case EmuConfig::L1:
            tabLibSidplay1->setEnabled(true);    
            break;
          case EmuConfig::L2:
             tabLibSidplay2->setEnabled(true);   
            break;            
          case EmuConfig::LFP:
            tabLibSidplayFp->setEnabled(true);    
            break;
        }  
    }
    setButtons();    
}

/*
 *  Destroys the object and frees any allocated resources
 */
EmuDialogNew::~EmuDialogNew()
{
    // no need to delete child widgets, Qt does it all for us
}


void EmuDialogNew::setConfig(const EmuConfig& inConfig)
{
    // Copy without verification.
    config = inConfig;
    
    // Make sure we do not send values because we are the receiver.
    enableChangedSignal(false);
    setButtons();
    enableChangedSignal(true);
}

void EmuDialogNew::receiveConfig(const EmuConfig& inConfig)
{
#ifdef SID_WITH_SIDPLAY1
    // Check flags that can be set from other dialogs.
    if (config.pConfig1->emulateFilter != inConfig.pConfig1->emulateFilter)
    {
        config = inConfig;
        // Make sure we do not send values because we are the receiver.
        enableChangedSignal(false);
        SIDfilter->setChecked(inConfig.pConfig1->emulateFilter);
        SIDfilter->repaint();
        enableChangedSignal(true);
    }
#endif    
}

const EmuConfig& EmuDialogNew::getConfig() const
{
    return config;
}


void EmuDialogNew::setButtons()
{
#ifdef SID_WITH_SIDPLAYFP
    // resid
    mos8580_LFp_Re->setChecked(config.pConfig3->mos8580_Re);
    mos6581_LFp_Re->setChecked(!config.pConfig3->mos8580_Re); 
    mos8580d_LFp_Re->setChecked(config.pConfig3->digibooster_Re);
    forceSM_LFp_Re->setChecked(config.pConfig3->forceSidModel_Re);
    
    SIDfilter_LFp_Re->setChecked(config.pConfig3->emulateFilter_Re);   
    forceSS_LFp_Re->setChecked(config.pConfig3->forceSongSpeed_Re);
    
    fullBS_LFp_Re->setChecked(config.pConfig3->memoryMode_Re==MPU_BANK_SWITCHING_L3);
    transROM_LFp_Re->setChecked(config.pConfig3->memoryMode_Re==MPU_TRANSPARENT_ROM_L3);
    psidEnv_LFp_Re->setChecked(config.pConfig3->memoryMode_Re==MPU_PLAYSID_ENVIRONMENT_L3);
    realEnv_LFp_Re->setChecked(config.pConfig3->memoryMode_Re==MPU_REALSID_ENVIRONMENT_L3);
    
    clockPAL_LFp_Re->setChecked(config.pConfig3->clockSpeed_Re==SIDTUNE_CLOCK_PAL);
    clockNTSC_LFp_Re->setChecked(config.pConfig3->clockSpeed_Re==SIDTUNE_CLOCK_NTSC);
    
    if (forceSS_LFp_Re->state()) clockGrp_LFp_Re->setEnabled(true);
    else clockGrp_LFp_Re->setEnabled(false);   
    
    if (forceSM_LFp_Re->state()) sidModelGrp_Lfp_Re->setEnabled(true);
    else sidModelGrp_Lfp_Re->setEnabled(false); 
    
    sampFast_LFp_Re->setChecked(config.pConfig3->sampling_Re==SAMPLE_FAST_L3);
    sampInterp_LFp_Re->setChecked(config.pConfig3->sampling_Re==SAMPLE_INTERPOLATE_L3);
    sampRes_LFp_Re->setChecked(config.pConfig3->sampling_Re==SAMPLE_RESAMPLE_L3);
    sampResFast_LFp_Re->setChecked(config.pConfig3->sampling_Re==SAMPLE_RESAMPLE_FASTMEM_L3);
    
    passband_LFp_Re->setValue(config.pConfig3->passbandPerc_Re);
    filterGain_LFp_Re->setValue(config.pConfig3->filterGainPerc_Re);
    filterBias_LFp_Re->setValue(config.pConfig3->filterBias_Re);    
    
    radioButton_LFp_Fp->setChecked(config.pConfig3->filterBias_Re==ENGINE_RESIDFP_L3);
    radioButton_LFp_Re->setChecked(config.pConfig3->filterBias_Re==ENGINE_RESID_L3);
    radioButton_LFp_Ha->setChecked(config.pConfig3->filterBias_Re=ENGINE_HARDSID_L3);     
#endif
  
#ifdef SID_WITH_SIDPLAY2
    SIDfilter->setChecked(config.pConfig2->emulateFilter);
    mos8580->setChecked(config.pConfig2->mos8580);
    forceSM->setChecked(config.pConfig2->forceSidModel);
    mos6581->setChecked(!config.pConfig2->mos8580);

    ///measVol->setChecked(config.measuredVolume);

    forceSS->setChecked(config.pConfig2->forceSongSpeed);
    
    fullBS->setChecked(config.pConfig2->memoryMode==MPU_BANK_SWITCHING);
    transROM->setChecked(config.pConfig2->memoryMode==MPU_TRANSPARENT_ROM);
    psidEnv->setChecked(config.pConfig2->memoryMode==MPU_PLAYSID_ENVIRONMENT);

    clockPAL->setChecked(config.pConfig2->clockSpeed==SIDTUNE_CLOCK_PAL);
    clockNTSC->setChecked(config.pConfig2->clockSpeed==SIDTUNE_CLOCK_NTSC);
  
    
    if (forceSM->state()) sidModelGrp->setEnabled(true);
    else sidModelGrp->setEnabled(false);
    
    if (forceSS->state()) clockGrp->setEnabled(true);
    else clockGrp->setEnabled(false);
#endif  
    
    // Activate startup value in driver box.
    int i = 0;
    while ( i < libraryBox->count() )
    {
        if ( libraryBox->itemText(i) == SidDrivers::getName() )
        {
            libraryBox->setCurrentIndex(i);
            break;
        }
        ++i;
    }    
}

void EmuDialogNew::enableChangedSignal(bool val)
{
    changedSignalEnabled = val;
}

void EmuDialogNew::emitChangedIfEnabled()
{ 
 #ifdef SID_WITH_SIDPLAY2 
    if (forceSM->state()) sidModelGrp->setEnabled(true);
    else sidModelGrp->setEnabled(false);
    
    if (forceSS->state()) clockGrp->setEnabled(true);
    else clockGrp->setEnabled(false);  
#endif
    
#ifdef SID_WITH_SIDPLAYFP  
    if (forceSM_LFp_Re->state()) sidModelGrp_Lfp_Re->setEnabled(true);
    else sidModelGrp_Lfp_Re->setEnabled(false);    
    
    if (forceSS_LFp_Re->state()) clockGrp_LFp_Re->setEnabled(true);
    else clockGrp_LFp_Re->setEnabled(false);     
#endif
  
    if (changedSignalEnabled)
        emit changed(config);
}

void EmuDialogNew::setLibrary(QString library) {
  if (library==LIB_EMPTY) config.library=0;
  if (library==LIB_L1) config.library=1;
  if (library==LIB_L2) config.library=2;
  if (library==LIB_LFP) config.library=3;
  
  SidDrivers::setLibrary(config.library);
  
  emitChangedIfEnabled();
}

void EmuDialogNew::newCLKsetting()
{   
#ifdef SID_WITH_SIDPLAY2 
  if (clockPAL->isChecked())
        config.pConfig2->clockSpeed = SIDTUNE_CLOCK_PAL;
    else  // if (clockNTSC->isChecked())
        config.pConfig2->clockSpeed = SIDTUNE_CLOCK_NTSC;
    emitChangedIfEnabled();
#endif
}

void EmuDialogNew::newEMUsetting()
{
#ifdef SID_WITH_SIDPLAY2  
    config.pConfig2->emulateFilter = SIDfilter->isChecked();
    config.pConfig2->forceSidModel = forceSM->isChecked();
    config.pConfig2->forceSongSpeed = forceSS->isChecked();
    emitChangedIfEnabled();  
#endif
}

void EmuDialogNew::newMPUsetting()
{
#ifdef SID_WITH_SIDPLAY2  
    if (fullBS->isChecked())
        config.pConfig2->memoryMode = MPU_BANK_SWITCHING;
    else if (transROM->isChecked())
        config.pConfig2->memoryMode = MPU_TRANSPARENT_ROM;
    else  // if (psidEnv->isChecked())
        config.pConfig2->memoryMode = MPU_PLAYSID_ENVIRONMENT;
    emitChangedIfEnabled();
#endif    
}

void EmuDialogNew::newSIDsetting()
{
#ifdef SID_WITH_SIDPLAY2
    config.pConfig2->mos8580 = mos8580->isChecked();
    emitChangedIfEnabled();
#endif  
}

void EmuDialogNew::mixer_L1()
{
#ifdef SID_WITH_SIDPLAY1
  myMixerDlg->show();
#endif  
}
  
void EmuDialogNew::filter_L1()
{
#ifdef SID_WITH_SIDPLAY1
  myFilterDlg->show();
#endif  
}


// libsidplayfp
// resid

void EmuDialogNew::newMPUsetting_LFp_Re()
{
#ifdef SID_WITH_SIDPLAYFP  
    if (fullBS_LFp_Re->isChecked())
        config.pConfig3->memoryMode_Re = MPU_BANK_SWITCHING_L3;
    else if (transROM_LFp_Re->isChecked())
        config.pConfig3->memoryMode_Re = MPU_TRANSPARENT_ROM_L3;
    else  if (psidEnv_LFp_Re->isChecked())
        config.pConfig3->memoryMode_Re = MPU_PLAYSID_ENVIRONMENT_L3;
    else 
        config.pConfig3->memoryMode_Re = MPU_REALSID_ENVIRONMENT_L3;
    emitChangedIfEnabled();
#endif    
}

void EmuDialogNew::newEMUsetting_LFp_Re()
{
#ifdef SID_WITH_SIDPLAYFP  
    config.pConfig3->emulateFilter_Re = SIDfilter_LFp_Re->isChecked();
    config.pConfig3->forceSidModel_Re = forceSM_LFp_Re->isChecked();
    config.pConfig3->forceSongSpeed_Re = forceSS_LFp_Re->isChecked();
    emitChangedIfEnabled();  
#endif    
}

void EmuDialogNew::newCLKsetting_LFp_Re()
{   
#ifdef SID_WITH_SIDPLAYFP
  if (clockPAL_LFp_Re->isChecked())
        config.pConfig3->clockSpeed_Re = SIDTUNE_CLOCK_PAL;
    else  // if (clockNTSC->isChecked())
        config.pConfig3->clockSpeed_Re = SIDTUNE_CLOCK_NTSC;
    emitChangedIfEnabled();
#endif
}

void EmuDialogNew::newSIDsetting_LFp_Re()
{
#ifdef SID_WITH_SIDPLAYFP
    config.pConfig3->mos8580_Re = (mos8580_LFp_Re->isChecked() || mos8580d_LFp_Re->isChecked());
    config.pConfig3->digibooster_Re = mos8580d_LFp_Re->isChecked();
    emitChangedIfEnabled();
#endif   
}

void EmuDialogNew::newSamplingSetting_LFp_Re()
{
#ifdef SID_WITH_SIDPLAYFP
  if (sampFast_LFp_Re->isChecked()) config.pConfig3->sampling_Re=SAMPLE_FAST_L3;
  if (sampInterp_LFp_Re->isChecked()) config.pConfig3->sampling_Re=SAMPLE_INTERPOLATE_L3;
  if (sampRes_LFp_Re->isChecked())config.pConfig3->sampling_Re=SAMPLE_RESAMPLE_L3;
  if (sampResFast_LFp_Re->isChecked()) config.pConfig3->sampling_Re=SAMPLE_RESAMPLE_FASTMEM_L3;  
#endif  
}

void EmuDialogNew::newPassbandSetting_LFp_Re(int value)
{
#ifdef SID_WITH_SIDPLAYFP
  config.pConfig3->passbandPerc_Re=value;
#endif  
}

void EmuDialogNew::newFilterGainSetting_LFp_Re(int value)
{
#ifdef SID_WITH_SIDPLAYFP
  config.pConfig3->filterGainPerc_Re=value;
#endif  
}

void EmuDialogNew::newFilterBiasSetting_LFp_Re(int value)
{
#ifdef SID_WITH_SIDPLAYFP
  config.pConfig3->filterBias_Re=value;
#endif  
}

void EmuDialogNew::newEngine_LFp()
{
#ifdef SID_WITH_SIDPLAYFP
  if (radioButton_LFp_Fp->isChecked()) config.pConfig3->filterBias_Re=ENGINE_RESIDFP_L3;
  if (radioButton_LFp_Re->isChecked()) config.pConfig3->filterBias_Re=ENGINE_RESID_L3;
  if (radioButton_LFp_Ha->isChecked()) config.pConfig3->filterBias_Re=ENGINE_HARDSID_L3;     
#endif  
}