/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <qcheckbox.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qslider.h>

#include "FilterDialog.h"
#include "GlobalIcon.h"
#include "images/filticon.xpm"
#include "wrapper/sid/EmuConfigImpl.h"

FilterDialog::FilterDialog(QWidget* parent)
: FilterDialogData(parent)
{
    setWindowIcon(*mainIcon);

    // TODO: move into .ui
    ///setTabOrder( isEnabled, para1Sld );
    ///setTabOrder( para1Sld, para2Sld );
    ///setTabOrder( para2Sld, para3Sld );
    ///setTabOrder( para3Sld, defaultBtn );

    enableChangedSignal(true);
    ///setButtons();
    paintFilterFunc();
}

void FilterDialog::enableChangedSignal(bool val)
{
    changedSignalEnabled = val;
}

void FilterDialog::emitChangedIfEnabled()
{
    if (changedSignalEnabled)
    {
        emit changed(config);
    }
}

void FilterDialog::setConfig(const EmuConfig& inConfig)
{
    enableChangedSignal(false);
    config = inConfig;
    ///setButtons();
    enableChangedSignal(true);
}

void FilterDialog::setConfigDefault(const EmuConfig& inConfig)
{
    configDefault = inConfig;
}

void FilterDialog::receiveConfig(const EmuConfig& inConfig)
{
#ifdef SID_WITH_SIDPLAY1  
    // Check flags that can be set from other dialogs.
    if (config.pConfig1->emulateFilter != inConfig.pConfig1->emulateFilter)
    {
        config = inConfig;
        // Make sure we do not send values because we are the receiver.
        enableChangedSignal(false);
        isEnabled->setChecked(inConfig.pConfig1->emulateFilter);
        isEnabled->repaint();
        enableChangedSignal(true);
    }
#endif    
}

void FilterDialog::setButtons()
{
#ifdef SID_WITH_SIDPLAY1
    isEnabled->setChecked(config.pConfig1->emulateFilter);
    setSliderValues();
#endif
}

void FilterDialog::setSliderValues()
{
#ifdef SID_WITH_SIDPLAY1  
    para1Sld->setValue((int)((config.pConfig1->filterFs/6.0-1.0)));
    para2Sld->setValue((int)(config.pConfig1->filterFm-1.0));
    para3Sld->setValue((int)(config.pConfig1->filterFt*100.0));
#endif    
}

void FilterDialog::updatePara1(int index)
{
#ifdef SID_WITH_SIDPLAY1 
    config.pConfig1->filterFs = 1.0+index*6.0;
    emitChangedIfEnabled();
#endif    
}

void FilterDialog::updatePara2(int index)
{
#ifdef SID_WITH_SIDPLAY1  
    config.pConfig1->filterFm = 1.0+index;
    emitChangedIfEnabled();
#endif    
}

void FilterDialog::updatePara3( int index )
{
#ifdef SID_WITH_SIDPLAY1
    config.pConfig1->filterFt = index/100.0;
    emitChangedIfEnabled();
#endif    
}

void FilterDialog::setDefaults()
{
#ifdef SID_WITH_SIDPLAY1 
    // Don't copy the full emuConfig because the rest of the default
    // doesn't get updated.
    config.pConfig1->emulateFilter = configDefault.pConfig1->emulateFilter;
    config.pConfig1->filterFs = configDefault.pConfig1->filterFs;
    config.pConfig1->filterFm = configDefault.pConfig1->filterFm;
    config.pConfig1->filterFt = configDefault.pConfig1->filterFt;

    setButtons();
    emitChangedIfEnabled();
#endif    
}

void FilterDialog::setFilterFlag(bool val)
{
#ifdef SID_WITH_SIDPLAY1 
    if (val != config.pConfig1->emulateFilter)
    {
        config.pConfig1->emulateFilter = val;
        emitChangedIfEnabled();
    }
#endif    
}

void FilterDialog::paintFilterFunc()
{
    QPixmap myFiltPic(filticon);
    QPainter paintBuf;
    paintBuf.begin(&myFiltPic);
    QColor myPenColor(0,0,255);  // rgb
    paintBuf.setPen(myPenColor);
#ifdef SID_WITH_SIDPLAY1
    extern float filterTable[0x800];  // the only way
    for (int i=0; i<0x800; i++)
    {
        paintBuf.drawPoint((int)(i/15.75),(int)(filterTable[i]*100.0));
    }
#endif
    paintBuf.end();

    //bitBlt( FiltPicFrame, 0, 0,
    //        &myFiltPic, 0, 0, myFiltPic.width(), myFiltPic.height(),
    //        Qt::CopyROP, false );
    
    QRectF rect(0.0, 0.0, myFiltPic.width(), myFiltPic.height());
    QPixmap pixmap(":myPixmap.png");

    QPainter painter(&myFiltPic);
    painter.drawPixmap(rect, pixmap, rect);
}

void FilterDialog::paintEvent(QPaintEvent *)
{
    paintFilterFunc();
}
