/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef EMUWRAPPER_H
#define EMUWRAPPER_H

#include "Config.h"
#include <qstring.h>
#include <qthread.h>
#include <qmutex.h>
#include "sid/SidDrivers.h"

struct EmuConfig;
class emuEngine;
class SidLibrary;

#include "TypeWrapper.h"

class EmuWrapper
{
 public:
    EmuWrapper();
    ~EmuWrapper();
    
    // thread-safe
    static const EmuConfig& getConfig();
    static const EmuConfig& getConfigDefault();
    static void setConfig(const EmuConfig&);
    static void initSong(int);
    static void fillBuffer(SidLibrary*, ubyte_emuwt* pBuffer, udword_emuwt bufferSize);
    static void stop();
    static void setVoiceVolume(int, ubyte_emuwt, ubyte_emuwt, uword_emuwt);
    static void resetSecondsThisSong();
    static int getSecondsThisSong();
    static int getSecondsTotal();
    static const char* getVersionString();
    static const char* getSpeedString();
    static const char* getSidModelString();
    static const char* getClockFreqString();
    static int getCia1TA();
    static int getClockSpeed();
   
 protected:
    static QString versionString;
    
    static int engines;
    static QMutex emuMutex;
    static EmuConfig myEmuConfig;
    static EmuConfig myEmuConfigDefault;
    
 private:
    // prevent copying
    EmuWrapper(const EmuWrapper&);
    EmuWrapper& operator=(EmuWrapper&);  
};

#endif  /* EMUWRAPPER_H */
