/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef EMUCONFIG
#define EMUCONFIG

#include <cstring>
#include "Config.h"
#include "EmuConfig.h"

#ifdef SID_WITH_SIDPLAYFP
#include "../emucfg3.h"
#endif

#ifdef SID_WITH_SIDPLAY2
#include "../emucfg2.h"
#endif

#ifdef SID_WITH_SIDPLAY1
#include "../emucfg.h"
#endif


EmuConfig::EmuConfig() {
  library = EMPTY;   
#ifdef SID_WITH_SIDPLAY1
  pConfig1=new emuConfig();
#endif  
#ifdef SID_WITH_SIDPLAY2 
  pConfig2=new emuConfig2();   
#endif
#ifdef SID_WITH_SIDPLAYFP 
  pConfig3=new emuConfig3();   
#endif  
}

EmuConfig::~EmuConfig() {
#ifdef SID_WITH_SIDPLAY1
  if (pConfig1!=NULL) {
    delete pConfig1;
    pConfig1=NULL;
  }  
#endif 
#ifdef SID_WITH_SIDPLAY2 
  if (pConfig2!=NULL) {
    delete pConfig2; 
    pConfig2=NULL;
  }  
#endif  
#ifdef SID_WITH_SIDPLAYFP 
  if (pConfig3!=NULL) {
    delete pConfig3; 
    pConfig3=NULL;
  }  
#endif 
}    

EmuConfig::EmuConfig(const EmuConfig& p) {
  copy(p);
}

void EmuConfig::copy(const EmuConfig& p) {
  library=p.library;
  
  // copy constructor 
#ifdef SID_WITH_SIDPLAY1
  pConfig1=new emuConfig();
  pConfig1->autoPanning = p.pConfig1->autoPanning;
  pConfig1->bitsPerSample = p.pConfig1->bitsPerSample;
  pConfig1->channels = p.pConfig1->channels;
  pConfig1->clockSpeed = p.pConfig1->clockSpeed;
  pConfig1->digiPlayerScans = p.pConfig1->digiPlayerScans;
  pConfig1->emulateFilter = p.pConfig1->emulateFilter;
  pConfig1->filterFm = p.pConfig1->filterFm;
  pConfig1->filterFs = p.pConfig1->filterFs; 
  pConfig1->filterFt = p.pConfig1->filterFt;
  pConfig1->forceSongSpeed = p.pConfig1->forceSongSpeed;
  pConfig1->frequency = p.pConfig1->frequency;
  pConfig1->measuredVolume = p.pConfig1->measuredVolume;
  pConfig1->memoryMode = p.pConfig1->memoryMode;
  pConfig1->mos8580 = p.pConfig1->mos8580;
  pConfig1->sampleFormat = p.pConfig1->sampleFormat;
  pConfig1->sidChips = p.pConfig1->sidChips;
  pConfig1->volumeControl = p.pConfig1->volumeControl;
#endif 
#ifdef SID_WITH_SIDPLAY2
  pConfig2=new emuConfig2(); 
  pConfig2->autoPanning = p.pConfig2->autoPanning;
  pConfig2->bitsPerSample = p.pConfig2->bitsPerSample;
  pConfig2->channels = p.pConfig2->channels;
  pConfig2->clockSpeed = p.pConfig2->clockSpeed;
  pConfig2->digiPlayerScans = p.pConfig2->digiPlayerScans;
  pConfig2->emulateFilter = p.pConfig2->emulateFilter;
  pConfig2->filterFm = p.pConfig2->filterFm;
  pConfig2->filterFs = p.pConfig2->filterFs;
  pConfig2->filterFt = p.pConfig2->filterFt;
  pConfig2->forceSidModel = p.pConfig2->forceSidModel;
  pConfig2->forceSongSpeed = p.pConfig2->forceSongSpeed;
  pConfig2->frequency = p.pConfig2->frequency;
  pConfig2->measuredVolume = p.pConfig2->measuredVolume;
  pConfig2->memoryMode = p.pConfig2->memoryMode;
  pConfig2->mos8580 = p.pConfig2->mos8580;
  pConfig2->sampleFormat = p.pConfig2->sampleFormat;
  pConfig2->sidChips = p.pConfig2->sidChips;
  pConfig2->volumeControl = p.pConfig2->volumeControl;
#endif   
#ifdef SID_WITH_SIDPLAYFP
  pConfig3=new emuConfig3(); 
  
  // resid
  pConfig3->memoryMode_Re = p.pConfig3->memoryMode_Re;
  pConfig3->emulateFilter_Re = p.pConfig3->emulateFilter_Re; 
  pConfig3->forceSongSpeed_Re = p.pConfig3->forceSongSpeed_Re;  
  pConfig3->clockSpeed_Re = p.pConfig3->clockSpeed_Re;  
  pConfig3->mos8580_Re = p.pConfig3->mos8580_Re;
  pConfig3->digibooster_Re = p.pConfig3->digibooster_Re; 
  pConfig3->forceSidModel_Re = p.pConfig3->forceSidModel_Re;
  pConfig3->sampling_Re = p.pConfig3->sampling_Re;
  pConfig3->passbandPerc_Re = p.pConfig3->passbandPerc_Re;
  pConfig3->filterGainPerc_Re = p.pConfig3->filterGainPerc_Re;
  pConfig3->filterBias_Re = p.pConfig3->filterBias_Re;  
  
  pConfig3->engine = p.pConfig3->engine;
  
  pConfig3->autoPanning = p.pConfig3->autoPanning;
  pConfig3->bitsPerSample = p.pConfig3->bitsPerSample;
  pConfig3->channels = p.pConfig3->channels;
  pConfig3->digiPlayerScans = p.pConfig3->digiPlayerScans;
  pConfig3->filterFm = p.pConfig3->filterFm;
  pConfig3->filterFs = p.pConfig3->filterFs;
  pConfig3->filterFt = p.pConfig3->filterFt;
  pConfig3->frequency = p.pConfig3->frequency;
  pConfig3->measuredVolume = p.pConfig3->measuredVolume;
  pConfig3->sampleFormat = p.pConfig3->sampleFormat;
  pConfig3->sidChips = p.pConfig3->sidChips;
  pConfig3->volumeControl = p.pConfig3->volumeControl;
#endif    
}

EmuConfig& EmuConfig::operator=(const EmuConfig& p) {
    // deep copy
    if (this != &p) {  // make sure not same object
#ifdef SID_WITH_SIDPLAY1
      if (pConfig1!=NULL) {
        delete pConfig1;
        pConfig1=NULL;
      }
#endif 
#ifdef SID_WITH_SIDPLAY2 
      if (pConfig2!=NULL) {
        delete pConfig2; 
        pConfig2=NULL;
      }
#endif
#ifdef SID_WITH_SIDPLAYFP 
      if (pConfig3!=NULL) {
        delete pConfig3; 
        pConfig3=NULL;
      }
#endif

      copy(p);
    }
    return *this;    // Return ref for multiple assignment  
}

#endif  /* EmuConfig */
