/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LIBNULL
#define LIBNULL

#include "LibNull.h"

/**
 * Construct the library
 */ 
LibNull::LibNull() {
}

/**
 * Deconstruct the library
 */ 
LibNull::~LibNull() {
}

/**
 * Get configuration
 * 
 * @return the configuration
 */ 
const EmuConfig& LibNull::getConfig() {
  return myEmuConfig; 
}
    
/**
 * Get default configuration
 * 
 * @return the default configuration
 */
const EmuConfig& LibNull::getConfigDefault() {
  return myEmuConfigDefault;  
}

/**
 * Set the configuration to use
 * 
 * @param inConfig input configuration
 */ 
void LibNull::setConfig(const EmuConfig& inConfig) {
  myEmuConfig=inConfig;
}

/**
 * Init with the given song
 * 
 * @param song the given number of song to init
 */ 
void LibNull::initSong(int song) {
}

/**
 * Fill the buffer with played music
 * 
 * @param pBuffer the buffer to fill
 * @param bufferSize the size of the buffer
 */ 
void LibNull::fillBuffer(void* pBuffer, unsigned long int bufferSize) {
}

/**
 * Set the volume for the given voice
 * 
 * @param voice the number of voice
 * @param left left volume
 * @param right right volume
 * @param total total volume
 */
void LibNull::setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total) {
}  

/**
 * Reset the seconds of this song
 */
void LibNull::resetSecondsThisSong() {  
}

/**
 * Get the seconds of this song
 * 
 * @return the seconds of this song
 */    
int LibNull::getSecondsThisSong() {
  return 0;
}
 
/**
 * Get the total seconds
 * 
 * @return the total seconds
 */    
int LibNull::getSecondsTotal() {
  return 0;
}      

/**
 * Get string version
 * 
 * @return the string version
 */
const char* LibNull::getVersionString() {
  versionString = "No Sid library used";
 
  return versionString.toLatin1();
}

/**
 * Get speed string
 * 
 * @return the speed string
 */
const char* LibNull::getSpeedString() {  
  return "";
}

/**
 * Get sid model string
 * 
 * @return the sid model string
 */    
const char* LibNull::getSidModelString() {
  return "";
}

/**
 * Get clock frequency string
 * 
 * @return the clock frequency string
 */      
const char* LibNull::getClockFreqString() {
  return "";  
} 
 
/**
 * Stop the engine
 */
void LibNull::stop() {
}  

/**
 * Get last CIA1 TA value of C64
 * 
 * @return the last CIA1 TA value
 */    
int LibNull::getCia1TA() {
  return -1;
}

/**
 * Get clock speed     
 *  
 * @return the clock speed (-1=not implemented)
 */
int LibNull::getClockSpeed() { 
  return -1;
}

/**
 * Get the max songs
 * 
 * @return the max songs
 */ 
int LibNull::getMaxSongs() const {
  return 0;
}

/**
 * Get the max sid file length
 * 
 * @return the max sid file length
 */ 
unsigned long int LibNull::getMaxSidFileLen() const {
  return 0;
}

    
/**
 * Open the give file
 * 
 * @param fileName the name of file
 * @return true if operation is ok 
 */ 
bool LibNull::open(const QString& fileName) {
  return true;  
}

/**
 * Load the file in the given buffer
 * 
 * @paraM buf the buffer
 * @param bufLen the length of buffer
 * @return true if the operation is ok
 */ 
bool LibNull::load(void* buf, unsigned long int bufLen) {
  return true;  
}

/**
 * Save the file as PSID
 * 
 * @param fileName the fileName to used
 * @param overWrite true if over write
 * @return true if the operation is ok
 */ 
bool LibNull::savePSID(const QString& fileName, bool overWrite) {
  return true;
}

/**
 * Update information
 */ 
void LibNull::updateInfo() {
}  

/**
 * Get the load address
 * 
 * @return the load address
 */ 
int LibNull::getLoadAddr() const {
  return 0;  
}

/**
 * Get the init address
 * 
 * @return the init address
 */
int LibNull::getInitAddr() const {
  return 0;
}

/**
 * Get the play address
 * 
 * @return the play address
 */ 
int LibNull::getPlayAddr() const {
  return 0;  
}

/**
 * Get the number of songs
 * 
 * @return the number of songs
 */ 
int LibNull::getSongs() const {
  return 0;
}

/**
 * Get the current song
 * 
 * @return the current song
 */ 
int LibNull::getCurrentSong() const {
  return 0;
}

/**
 * Get thwe number of starting song
 * 
 * @return the starting song
 */ 
int LibNull::getStartSong() const {
  return 0;
}

/**
 * Get the info string of the given index
 * 
 * @param i the index
 * @return the info string
 */ 
const char* LibNull::getInfoString(int i) const {
  return "";
}

/**
 * Get numebr of info string
 * 
 * @return the number of input string
 */ 
int LibNull::getInfoStringsNum() const {
  return 0;
}

/**
 * Get the status string
 * 
 * @return the status string
 */ 
const char* LibNull::getStatusString() const {
  return "";
}

/**
 * Get the speed string
 * 
 * @return the speed string
 */
const char* LibNull::getSpeedString2() const {
  return "";
}

/**
 * Get the format string
 * 
 * @return the formal string
 */
const char* LibNull::getFormatString() const {    
  return "";
}

/**
 * Get status
 * 
 * @return the status
 */ 
bool LibNull::getStatus() {
  return true;
}

/**
 * Get MD5
 * 
 * @return the MD5
 */    
const QString& LibNull::getMD5_Digest() {
  const QString* tmp=new QString("00000000000000000000000000000000");
  return *tmp; 
}

/**
 * Create MD5
 * 
 * @param myMD5 MD5 to create
 */
void LibNull::createMD5(MD5& myMD5) {
} 

/**
 * Get the data buffer
 * 
 * @return the data buffer
 */ 
const unsigned char *LibNull::getDataBuffer() {
  return NULL;
}

/**
 * Get the data length
 * 
 * @return the data length
 */
int LibNull::getDataLength() {
  return 0;
}

#endif  /* LIBNULL */