/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "SidDrivers.h"
#include "SidLibrary.h"

#ifdef XSID_WB_DEBUG
#include <iostream>
#include <iomanip>
using namespace std;
#endif

SidDrivers::List SidDrivers::driverList;

SidDrivers::Entry SidDrivers::theDriver;

SidDrivers::SidDrivers()
{
}

SidDrivers::~SidDrivers()
{ 
  dispose();
}

// dispose memory used
void SidDrivers::dispose() {
  while ( !driverList.empty() )
    {
        delete driverList.back().pDriver;
#ifdef XSID_WB_DEBUG
        cout << "ADR::~ deleting: " << driverList.back().name.ascii() << endl;
#endif
        driverList.pop_back();
    }
}

void SidDrivers::setDriver( const QString& name )
{
    SidDrivers::List::iterator it;
    for ( it = SidDrivers::begin(); it != SidDrivers::end(); it++ )
    {
        if ( it->name == name )
        {
            theDriver = *it;
#ifdef XSID_WB_DEBUG
            cout << "ADR::setDriver: " << theDriver.name.ascii() << endl;
#endif
        }
    }
}

SidLibrary* SidDrivers::getDriver()
{
    return theDriver.pDriver;
}

const QString& SidDrivers::getName()
{
    return theDriver.name;
}

const int SidDrivers::getEnum()
{
    return theDriver.library;
}

void SidDrivers::addDriver( SidLibrary* p, bool isDefault )
{
    Entry e;
    e.pDriver = p;
    e.name = p->getName();
    e.library = p->getEnum();
    
    driverList.push_back( e );
#ifdef XSID_WB_DEBUG
    cout << "ADR::addDriver: " << e.name.ascii() << endl;
#endif
    if ( isDefault )
        theDriver = e;
}

SidDrivers::List::iterator SidDrivers::begin()
{
    return driverList.begin();
}

SidDrivers::List::iterator SidDrivers::end()
{
    return driverList.end();
}

void SidDrivers::setLibrary(const int library) 
{
    SidDrivers::List::iterator it;
    for ( it = SidDrivers::begin(); it != SidDrivers::end(); it++ )
    {
        if ( it->library == library )
        {
            theDriver = *it;
#ifdef XSID_WB_DEBUG
            cout << "ADR::setDriver: " << theDriver.name.ascii() << endl;
#endif
            return;
        }
    }
    
    // give default driver as the passed is not fount
    theDriver = *SidDrivers::begin();
}
